/////////////////////////////////////
// DS1631.cpp
// part of 720020-1174x (I2C)
//
// Applied Data Systems
// www.applieddata.net
//
// { See DS1631.h for more description and history }
/////////////////////////////////////
#include "ds1631.h"

DS1631::DS1631(BYTE busAddress) : I2C(busAddress)
{
	DEBUGMSG(TRUE, (_T("+DS1631::DS1631(0x%02x)\r\n"), busAddress));

	if(((busAddress >> 4) & 0x09) != 0x09)
	{
		RETAILMSG(TRUE, (_T("Invalid address for DS1631 part!\r\n")));
		delete this;
	}

	DEBUGMSG(TRUE, (_T("-DS1631::DS1631\r\n")));
}

void DS1631::WriteConfiguration(BYTE newConfig)
{
	DEBUGMSG(TRUE, (_T("+DS1631::WriteConfiguration(0x%02x)\r\n"), newConfig));

	BYTE data[3];

	// send the DS1631_ACCESS_CONFIG command
	data[0] = m_busAddress | I2C_WRITE;	// address as write
	data[1] = DS1631_ACCESS_CONFIG;		// command
	data[2] = newConfig;				// configuration

	Write(data, 3, TRUE, TRUE);

	DEBUGMSG(TRUE, (_T("-DS1631::WriteConfiguration\r\n")));
}

BYTE DS1631::ReadConfiguration()
{
	DEBUGMSG(TRUE, (_T("+DS1631::ReadConfiguration\r\n")));

	BYTE			data[3];

	// send the DS1631_ACCESS_CONFIG command
	data[0]		= m_busAddress | I2C_WRITE;	// address as write
	data[1]		= DS1631_ACCESS_CONFIG;		// command
	Write(data, 2, TRUE, FALSE);

	// tell the device we're going to read
	data[0]		= m_busAddress | I2C_READ;	// address as read
	Write(data, 1, TRUE, FALSE);

	// read data off the bus
	data[0]		= 0;	// clear the byte so we know the return is good
	Read(data, 1, FALSE, TRUE);

	DEBUGMSG(TRUE, (_T("-DS1631::ReadConfiguration [0x%02x]\r\n"), data[0]));

	return data[0];
}

float DS1631::ReadCurrentTempF()
{
	return (float)((ReadCurrentTempC() * 1.8) + 32);
}

float DS1631::ReadCurrentTempC()
{
	DEBUGMSG(TRUE, (_T("+DS1631::ReadCurrentTemp\r\n")));

	// reading the temp consists of a start (0x51) followed by a Read (0xAA)
	BYTE			data[3];

	// send the DS1631_START_CONVERSION command
	data[0]		= m_busAddress | I2C_WRITE;	// address as write
	data[1]		= DS1631_START_CONVERSION;  // command
	Write(data, 2, TRUE, TRUE);

	// in 10-bit resolution the DS1631 has a max conversion time of 750ms
	// so assume the worst case scenario
	Sleep(750);  

	// send the DS1631_READ_TEMP command
	data[0]		= m_busAddress | I2C_WRITE;	// address as write
	data[1]		= DS1631_READ_TEMP;			// command
	Write(data, 2, TRUE, FALSE);

	// tell the device we're going to read
	data[0]		= m_busAddress | I2C_READ;	// address as read
	Write(data, 1, TRUE, FALSE);

	// read data off the bus
	data[0]		= 0;	// clear the byte so we know the return is good
	data[1]		= 0;	// clear the byte so we know the return is good
	Read(data, 2, FALSE, TRUE);

	DEBUGMSG(TRUE, (_T("   Temp Data: 0x%02x%02x\r\n"), data[0], data[1]));

	WORD temp = data[0] + ((data[1] >> 4) / 15);

	DEBUGMSG(TRUE, (_T("-DS1631::ReadCurrentTemp [%i]\r\n"), temp));

	return temp;
}

void DS1631::PowerOnReset()
{
	DEBUGMSG(TRUE, (_T("+DS1631::PowerOnReset\r\n")));

	BYTE data[2];

	// send the DS1631_POR command
	data[0] = m_busAddress | I2C_WRITE;	// address as write
	data[1] = DS1631_POR;				// command

	Write(data, 2, TRUE, TRUE);

	DEBUGMSG(TRUE, (_T("-DS1631::PowerOnReset\r\n")));
}

void DS1631::SetTH(WORD th)
{
	DEBUGMSG(TRUE, (_T("+DS1631::SetTH\r\n")));
	
	BYTE data[4];

	// send the DS1631_ACCESS_TH command
	data[0] = m_busAddress | I2C_WRITE;	// address as write
	data[1] = DS1631_ACCESS_TH;				// command
	data[2] = ((BYTE*)&th)[0];
	data[3] = ((BYTE*)&th)[1];

	Write(data, 4, TRUE, TRUE);

	DEBUGMSG(TRUE, (_T("-DS1631::SetTH\r\n")));
}

WORD DS1631::GetTH()
{
	DEBUGMSG(TRUE, (_T("+DS1631::GetTH\r\n")));
	
	BYTE data[2];

	// send the DS1631_ACCESS_TH command
	data[0] = m_busAddress | I2C_WRITE;	// address as write
	data[1] = DS1631_ACCESS_TH;				// command

	Write(data, 2, TRUE, FALSE);

	// tell the device we're going to read
	data[0]		= m_busAddress | I2C_READ;	// address as read
	Write(data, 1, TRUE, FALSE);

	// read data off the bus
	data[0]		= 0;	// clear the byte so we know the return is good
	data[1]		= 0;	// clear the byte so we know the return is good
	Read(data, 2, FALSE, TRUE);

	DEBUGMSG(TRUE, (_T("-DS1631::GetTH\r\n")));

	return (WORD)*data;
}

void DS1631::SetTL(WORD tl)
{
	DEBUGMSG(TRUE, (_T("+DS1631::SetTL\r\n")));
	
	BYTE data[4];

	// send the DS1631_ACCESS_TL command
	data[0] = m_busAddress | I2C_WRITE;	// address as write
	data[1] = DS1631_ACCESS_TL;				// command
	data[2] = ((BYTE*)&tl)[0];
	data[3] = ((BYTE*)&tl)[1];

	Write(data, 4, TRUE, TRUE);

	DEBUGMSG(TRUE, (_T("-DS1631::SetTL\r\n")));
}

WORD DS1631::GetTL()
{
	DEBUGMSG(TRUE, (_T("+DS1631::GetTL\r\n")));
	
	BYTE data[2];

	// send the DS1631_ACCESS_TL command
	data[0] = m_busAddress | I2C_WRITE;	// address as write
	data[1] = DS1631_ACCESS_TL;				// command

	Write(data, 2, TRUE, FALSE);

	// tell the device we're going to read
	data[0]		= m_busAddress | I2C_READ;	// address as read
	Write(data, 1, TRUE, FALSE);

	// read data off the bus
	data[0]		= 0;	// clear the byte so we know the return is good
	data[1]		= 0;	// clear the byte so we know the return is good
	Read(data, 2, FALSE, TRUE);

	DEBUGMSG(TRUE, (_T("-DS1631::GetTH\r\n")));

	return (WORD)*data;
}